==> apollopay-1.0.1.tar.gz: lib/src/api/entities/payment/invoice_entity.dart <==
import 'dart:convert';

import 'package:apollopay/src/api/entities/client_entity.dart';
import 'package:apollopay/src/api/entities/entity_entity.dart';
import 'package:apollopay/src/api/entities/payment/charge_entity.dart';
import 'package:apollopay/src/api/entities/payment/customer_entity.dart';
import 'package:apollopay/src/api/entities/payment/pricing/price_entity.dart';
import 'package:collection/collection.dart';
import 'package:mongo_dart/mongo_dart.dart';

class Invoice extends Entity {
  String? id;
  String? token;
  String? clientId;
  String? customerToken;
  List<InvoiceLine> lines;
  Map<String, String>? metadata;
  List<String> taxesTokens;
  bool sendViaEmail;
  DateTime date;

  /// Assigned by the controller
  Client? client;

  /// Assigned by the controller
  Customer? customer;

  /// Assigned by the controller
  List<dynamic>? taxes;

  /// Assigned by the controller
  List<Charge>? charges;

  int get subTotal {
    if (lines.isEmpty) {
      return 0;
    }

    return lines.fold<int>(
        0,
        (previousValue, element) =>
            previousValue + (element.price!.amount * element.quantity));
  }

  int get total {
    if (taxesTokens.isEmpty || taxes?.isEmpty == true) {
      return subTotal;
    }
    // TODO:
    return subTotal;
  }

  Invoice(
      {this.id,
      this.token,
      this.clientId,
      this.customerToken,
      required this.lines,
      this.metadata,
      required this.taxesTokens,
      this.sendViaEmail = false,
      this.client,
      required this.date,
      this.customer,
      this.charges});

  @override
  Invoice copyWith(
      {String? clientId,
      String? customerToken,
      List<InvoiceLine>? lines,
      Map<String, String>? metadata,
      List<String>? taxesTokens,
      bool? sendViaEmail,
      DateTime? date,
      Client? client,
      Customer? customer}) {
    return Invoice(
        clientId: clientId ?? this.clientId,
        customerToken: customerToken ?? this.customerToken,
        lines: lines ?? this.lines,
        metadata: metadata ?? this.metadata,
        taxesTokens: taxesTokens ?? this.taxesTokens,
        sendViaEmail: sendViaEmail ?? this.sendViaEmail,
        date: date ?? this.date,
        client: client ?? this.client,
        token: token,
        customer: customer ?? this.customer,
        charges: charges);
  }

  @override
  Map<String, dynamic> toMap({bool withId = false, bool withPrices = true}) {
    return {
      if (withId) '_id': id,
      'token': token,
      'client_id': clientId,
      'customer_token': customerToken,
      'lines': lines.map((e) => e.toMap(withPrice: withPrices)).toList(),
      'metadata': metadata,
      'taxes_tokens': taxesTokens,
      'send_via_email': sendViaEmail,
      'date': date.millisecondsSinceEpoch,
      if (withPrices) 'client': client?.toMap(),
      if (withPrices) 'customer': customer?.toMap(),
      if (withPrices) 'charges': charges?.map((e) => e.toMap()).toList(),
    };
  }

  factory Invoice.fromMap(Map<String, dynamic> map) {
    return Invoice(
        id: map['_id'] is ObjectId
            ? map['_id'].toHexString()
            : map['_id'].toString(),
        token: map['token'],
        clientId: map['client_id'],
        customerToken: map['customer_token'],
        lines: List<InvoiceLine>.from(List<InvoiceLine>.from(
            map['lines'].map((e) => InvoiceLine.fromMap(e)))),
        metadata: (map['metadata'] != null)
            ? Map<String, String>.from(map['metadata'])
            : {},
        taxesTokens: List<String>.from(map['taxes_tokens']),
        sendViaEmail: map['send_via_email'].toString() == "true",
        date: DateTime.fromMillisecondsSinceEpoch(
            int.parse(map['date'].toString())),
        client: (map['client'] != null) ? Client.fromMap(map['client']) : null,
        customer: (map['customer'] != null)
            ? Customer.fromMap(map['customer'])
            : null,
        charges: (map['charges'] != null)
            ? List<Charge>.from(
                List<Charge>.from(map['charges'].map((e) => Charge.fromMap(e))))
            : null);
  }

  @override
  Map<String, dynamic> toJson() => (toMap());

  factory Invoice.fromJson(String source) =>
      Invoice.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Invoice(id: $id, token: $token, client: $clientId, customer: $customerToken, lines: $lines, metadata: $metadata, taxes: $taxesTokens)';
  }

  @override
  bool operator ==(Object? other) {
    if (identical(this, other)) return true;
    final collectionEquals = const DeepCollectionEquality().equals;

    return other is Invoice &&
        other.id == id &&
        other.token == token &&
        other.clientId == clientId &&
        other.customerToken == customerToken &&
        collectionEquals(other.lines, lines) &&
        collectionEquals(other.metadata, metadata) &&
        collectionEquals(other.taxesTokens, taxesTokens);
  }

  @override
  int get hashCode {
    return id.hashCode ^
        token.hashCode ^
        clientId.hashCode ^
        customerToken.hashCode ^
        lines.hashCode ^
        metadata.hashCode ^
        taxesTokens.hashCode;
  }
}

class InvoiceLine extends Entity {
  String priceToken;
  int quantity;

  // Assigned by the controller
  Price? price;

  InvoiceLine({required this.priceToken, required this.quantity, this.price});

  @override
  InvoiceLine copyWith({String? priceToken, int? quantity, Price? price}) {
    return InvoiceLine(
        priceToken: priceToken ?? this.priceToken,
        quantity: quantity ?? this.quantity,
        price: price ?? this.price);
  }

  @override
  Map<String, dynamic> toMap({withPrice = true}) {
    return {
      'price_token': priceToken,
      'quantity': quantity,
      if (price != null && withPrice) 'price': price
    };
  }

  factory InvoiceLine.fromMap(Map<String, dynamic> map) {
    return InvoiceLine(
        priceToken: map['price_token'] ?? '',
        quantity: map['quantity']?.toInt() ?? 0,
        price: (map['price'] != null) ? Price.fromMap(map['price']) : null);
  }

  @override
  Map<String, dynamic> toJson() => (toMap());

  factory InvoiceLine.fromJson(String source) =>
      InvoiceLine.fromMap(json.decode(source));

  @override
  String toString() =>
      'InvoiceLine(priceToken: $priceToken, quantity: $quantity)';

  @override
  bool operator ==(Object? other) {
    if (identical(this, other)) return true;

    return other is InvoiceLine &&
        other.priceToken == priceToken &&
        other.quantity == quantity;
  }

  @override
  int get hashCode => priceToken.hashCode ^ quantity.hashCode;
}

==> apollopay-1.0.6.tar.gz: lib/src/api/entities/payment/invoice_entity.dart <==
import 'dart:convert';

import 'package:apollopay/src/api/entities/client_entity.dart';
import 'package:apollopay/src/api/entities/clonable_entity.dart';
import 'package:apollopay/src/api/entities/entity_entity.dart';
import 'package:apollopay/src/api/entities/payment/charge_entity.dart';
import 'package:apollopay/src/api/entities/payment/customer_entity.dart';
import 'package:apollopay/src/api/entities/payment/pricing/price_entity.dart';
import 'package:collection/collection.dart';
import 'package:mongo_dart/mongo_dart.dart';

class Invoice extends Entity {
  String? id;
  String? token;
  String? clientId;
  String? customerToken;
  List<InvoiceLine> lines;
  Map<String, String>? metadata;
  List<String> taxesTokens;
  bool sendViaEmail;
  DateTime date;

  /// Assigned by the controller
  Client? client;

  /// Assigned by the controller
  Customer? customer;

  /// Assigned by the controller
  List<dynamic>? taxes;

  /// Assigned by the controller
  List<Charge>? charges;

  double get subTotal {
    if (lines.isEmpty) {
      return 0;
    }

    return lines.fold<double>(
        0,
        (previousValue, element) =>
            previousValue + (element.price!.amount * element.quantity));
  }

  double get total {
    if (taxesTokens.isEmpty || taxes?.isEmpty == true) {
      return subTotal;
    }
    // TODO:
    return subTotal;
  }

  Invoice(
      {this.id,
      this.token,
      this.clientId,
      this.customerToken,
      required this.lines,
      this.metadata,
      required this.taxesTokens,
      this.sendViaEmail = false,
      this.client,
      required this.date,
      this.customer,
      this.charges});

  @override
  Invoice copyWith(
      {String? clientId,
      String? customerToken,
      List<InvoiceLine>? lines,
      Map<String, String>? metadata,
      List<String>? taxesTokens,
      bool? sendViaEmail,
      DateTime? date,
      Client? client,
      Customer? customer}) {
    return Invoice(
        clientId: clientId ?? this.clientId,
        customerToken: customerToken ?? this.customerToken,
        lines: lines ?? this.lines,
        metadata: metadata ?? this.metadata,
        taxesTokens: taxesTokens ?? this.taxesTokens,
        sendViaEmail: sendViaEmail ?? this.sendViaEmail,
        date: date ?? this.date,
        client: client ?? this.client,
        token: token,
        customer: customer ?? this.customer,
        charges: charges);
  }

  @override
  Map<String, dynamic> toMap({bool withId = false, bool withPrices = true}) {
    return {
      if (withId) '_id': id,
      'token': token,
      'client_id': clientId,
      'customer_token': customerToken,
      'lines': lines.map((e) => e.toMap(withPrice: withPrices)).toList(),
      'metadata': metadata,
      'taxes_tokens': taxesTokens,
      'send_via_email': sendViaEmail,
      'date': date.millisecondsSinceEpoch,
      if (withPrices) 'client': client?.toMap(),
      if (withPrices) 'customer': customer?.toMap(),
      if (withPrices) 'charges': charges?.map((e) => e.toMap()).toList(),
    };
  }

  factory Invoice.fromMap(Map<String, dynamic> map) {
    return Invoice(
        id: map['_id'] is ObjectId
            ? map['_id'].toHexString()
            : map['_id'].toString(),
        token: map['token'],
        clientId: map['client_id'],
        customerToken: map['customer_token'],
        lines: List<InvoiceLine>.from(List<InvoiceLine>.from(
            map['lines'].map((e) => InvoiceLine.fromMap(e)))),
        metadata: (map['metadata'] != null)
            ? Map<String, String>.from(map['metadata'])
            : {},
        taxesTokens: List<String>.from(map['taxes_tokens']),
        sendViaEmail: map['send_via_email'].toString() == "true",
        date: DateTime.fromMillisecondsSinceEpoch(
            int.parse(map['date'].toString())),
        client: (map['client'] != null) ? Client.fromMap(map['client']) : null,
        customer: (map['customer'] != null)
            ? Customer.fromMap(map['customer'])
            : null,
        charges: (map['charges'] != null)
            ? List<Charge>.from(
                List<Charge>.from(map['charges'].map((e) => Charge.fromMap(e))))
            : null);
  }

  @override
  Map<String, dynamic> toJson() => (toMap());

  factory Invoice.fromJson(String source) =>
      Invoice.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Invoice(id: $id, token: $token, client: $clientId, customer: $customerToken, lines: $lines, metadata: $metadata, taxes: $taxesTokens)';
  }

  @override
  bool operator ==(Object? other) {
    if (identical(this, other)) return true;
    final collectionEquals = const DeepCollectionEquality().equals;

    return other is Invoice &&
        other.id == id &&
        other.token == token &&
        other.clientId == clientId &&
        other.customerToken == customerToken &&
        collectionEquals(other.lines, lines) &&
        collectionEquals(other.metadata, metadata) &&
        collectionEquals(other.taxesTokens, taxesTokens);
  }

  @override
  int get hashCode {
    return id.hashCode ^
        token.hashCode ^
        clientId.hashCode ^
        customerToken.hashCode ^
        lines.hashCode ^
        metadata.hashCode ^
        taxesTokens.hashCode;
  }
}

class InvoiceLine extends Entity {
  String priceToken;
  double? amount;
  String? description;
  double quantity;

  // Assigned by the controller
  Price? price;

  InvoiceLine(
      {required this.priceToken,
      required this.quantity,
      this.price,
      this.description,
      this.amount});

  InvoiceLine copyWith(
      {String? priceToken,
      double? quantity,
      Price? price,
      String? description,
      double? amount}) {
    return InvoiceLine(
        priceToken: priceToken ?? this.priceToken,
        quantity: quantity ?? this.quantity,
        description: description ?? this.description,
        amount: amount ?? this.amount,
        price: price ?? this.price);
  }

  @override
  Map<String, dynamic> toMap({bool withId = true, withPrice = true}) {
    return {
      'price_token': priceToken,
      'quantity': quantity,
      if (price != null && withPrice) 'price': price?.toMap(),
      'description': description,
      'amount': amount
    };
  }

  factory InvoiceLine.fromMap(Map<String, dynamic> map) {
    assert(
        (map['price_token'] != null && map['amount'] == null) ||
            (map['price_token'] == null && map['amount'] != null),
        'InvoiceError -> A price_token OR an amount must be set.');

    return InvoiceLine(
        priceToken: map['price_token'] ?? '',
        quantity: map['quantity']?.toInt() ?? 0,
        description: map['description'],
        amount: map['amount']?.toDouble(),
        price: (map['price'] != null) ? Price.fromMap(map['price']) : null);
  }

  factory InvoiceLine.fromJson(String source) =>
      InvoiceLine.fromMap(json.decode(source));

  @override
  String toString() =>
      'InvoiceLine(priceToken: $priceToken, quantity: $quantity)';

  @override
  bool operator ==(Object? other) {
    if (identical(this, other)) return true;

    return other is InvoiceLine &&
        other.priceToken == priceToken &&
        other.quantity == quantity;
  }

  @override
  int get hashCode => priceToken.hashCode ^ quantity.hashCode;

  @override
  Cloneable clone() {
    return InvoiceLine(
        priceToken: priceToken,
        quantity: quantity,
        price: price,
        description: description,
        amount: amount);
  }
}
